;;;; -*- Mode:Common-Lisp; Package:GBBOPEN-TOOLS; Syntax:common-lisp -*-
;;;; *-* File: /usr/local/gbbopen/source/tools/test/double-metaphone-test.lisp *-*
;;;; *-* Edited-By: cork *-*
;;;; *-* Last-Edit: Wed Apr  7 10:02:01 2010 *-*
;;;; *-* Machine: cyclone.cs.umass.edu *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *
;;;; *                          Double Metaphone Test
;;;; *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Dan Corkill
;;;
;;; Part of the GBBopen Project.
;;; Licensed under Apache License 2.0 (see LICENSE for license information).
;;;
;;; Incorporates data from various Double Metaphone test sets.  Dan Corkill
;;; added the `extended-p' extension tests.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  12-04-07 File Created.  (Corkill)
;;;  02-22-10 Added more extended-p extension tests.  (Corkill)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :gbbopen-tools)

;;; ---------------------------------------------------------------------------

(defparameter *encodings*
    '(("testing" "TSTN" nil)
      ("" "" nil)
      ("the" "0" "T")
      ("quick" "KK" nil)
      ("fox" "FKS" nil)
      ("jumped" "JMPT" "AMPT" "JMPD" "AMPD")
      ("over" "AFR" nil "AVR")
      ("lazy" "LS" nil "LZ")
      ("dogs" "TKS" nil "DKS")
      ("MacCafferey" "MKFR" nil)
      ("Stephan" "STFN" nil)
      ("Kuczewski" "KSSK" "KXFS")
      ("Kutchefski" "KXFS" nil)
      ("San Jose" "SNHS" nil)
      ("xenophobia" "SNFP" nil "SNFB")
      
      ;; From PHP3 tests:

      ("Allerton" "ALRT" nil)
      ("Acton" "AKTN" nil)
      ("Adams" "ATMS" nil "ADMS")
      ("Aggar" "AKR" nil)
      ("Ahl" "AL" nil)
      ("Aiken" "AKN" nil)
      ("Alan" "ALN" nil)
      ("Alcock" "ALKK" nil)
      ("Alden" "ALTN" nil "ALDN")
      ("Aldham" "ALTM" nil "ALDM")
      ("Allen" "ALN" nil)
      ("Allerton" "ALRT" nil)
      ("Alsop" "ALSP" nil)
      ("Alwein" "ALN" nil)
      ("Ambler" "AMPL" nil "AMBL")
      ("Andevill" "ANTF" nil "ANDV")
      ("Andrews" "ANTR" nil "ANDR")
      ("Andreyco" "ANTR" nil "ANDR")
      ("Andriesse" "ANTR" nil "ANDR")
      ("Angier" "ANJ" "ANJR")
      ("Annabel" "ANPL" nil "ANBL")
      ("Anne" "AN" nil)
      ("Anstye" "ANST" nil)
      ("Appling" "APLN" nil)
      ("Apuke" "APK" nil)
      ("Arnold" "ARNL" nil)
      ("Ashby" "AXP" nil "AXB")
      ("Astwood" "ASTT" nil "ASTD")
      ("Atkinson" "ATKN" nil)
      ("Audley" "ATL" nil "ADL")
      ("Austin" "ASTN" nil)
      ("Avenal" "AFNL" nil "AVNL")
      ("Ayer" "AR" nil)
      ("Ayot" "AT" nil)
      ("Babbitt" "PPT" nil "BBT")
      ("Bachelor" "PXLR" "PKLR" "BXLR" "BKLR")
      ("Bachelour" "PXLR" "PKLR" "BXLR" "BKLR")
      ("Bailey" "PL" nil "BL")
      ("Baivel" "PFL" nil "BVL")
      ("Baker" "PKR" nil "BKR")
      ("Baldwin" "PLTN" nil "BLDN")
      ("Balsley" "PLSL" nil "BLSL")
      ("Barber" "PRPR" nil "BRBR")
      ("Barker" "PRKR" nil "BRKR")
      ("Barlow" "PRL" "PRLF" "BRL" "BRLF")
      ("Barnard" "PRNR" nil "BRNR")
      ("Barnes" "PRNS" nil "BRNS")
      ("Barnsley" "PRNS" nil "BRNS")
      ("Barouxis" "PRKS" nil "BRKS")
      ("Bartlet" "PRTL" nil "BRTL")
      ("Basley" "PSL" nil "BSL")
      ("Basset" "PST" nil "BST")
      ("Bassett" "PST" nil "BST")
      ("Batchlor" "PXLR" nil "BXLR")
      ("Bates" "PTS" nil "BTS")
      ("Batson" "PTSN" nil "BTSN")
      ("Bayes" "PS" nil "BS")
      ("Bayley" "PL" nil "BL")
      ("Beale" "PL" nil "BL")
      ("Beauchamp" "PXMP" "PKMP" "BXMP" "BKMP")
      ("Beauclerc" "PKLR" nil "BKLR")
      ("Beech" "PK" nil "BK")
      ("Beers" "PRS" nil "BRS")
      ("Beke" "PK" nil "BK")
      ("Belcher" "PLXR" "PLKR" "BLXR" "BLKR")
      ("Benjamin" "PNJM" nil "BNJM")
      ("Benningham" "PNNK" nil "BNNK")
      ("Bereford" "PRFR" nil "BRFR")
      ("Bergen" "PRJN" "PRKN" "BRJN" "BRKN")
      ("Berkeley" "PRKL" nil "BRKL")
      ("Berry" "PR" nil "BR")
      ("Besse" "PS" nil "BS")
      ("Bessey" "PS" nil "BS")
      ("Bessiles" "PSLS" nil "BSLS")
      ("Bigelow" "PJL" "PKLF" "BJL" "BKLF")
      ("Bigg" "PK" nil "BK")
      ("Bigod" "PKT" nil "BKD")
      ("Billings" "PLNK" nil "BLNK")
      ("Bimper" "PMPR" nil "BMPR")
      ("Binker" "PNKR" nil "BNKR")
      ("Birdsill" "PRTS" nil "BRDS")
      ("Bishop" "PXP" nil "BXP")
      ("Black" "PLK" nil "BLK")
      ("Blagge" "PLK" nil "BLK")
      ("Blake" "PLK" nil "BLK")
      ("Blanck" "PLNK" nil "BLNK")
      ("Bledsoe" "PLTS" nil "BLDS")
      ("Blennerhasset" "PLNR" nil "BLNR")
      ("Blessing" "PLSN" nil "BLSN")
      ("Blewett" "PLT" nil "BLT")
      ("Bloctgoed" "PLKT" nil "BLKT")
      ("Bloetgoet" "PLTK" nil "BLTK")
      ("Bloodgood" "PLTK" nil "BLTK")
      ("Blossom" "PLSM" nil "BLSM")
      ("Blount" "PLNT" nil "BLNT")
      ("Bodine" "PTN" nil "BDN") 
      ("Bodman" "PTMN" nil "BDMN")
      ("BonCoeur" "PNKR" nil "BNKR")
      ("Bond" "PNT" nil "BND") 
      ("Boscawen" "PSKN" nil "BSKN")
      ("Bosworth" "PSR0" "PSRT" "BSR0" "BSRT")
      ("Bouchier" "PX" "PKR" "BX" "BKR")
      ("Bowne" "PN" nil "BN")
      ("Bradbury" "PRTP" nil "BRDB")
      ("Bradder" "PRTR" nil "BRDR")
      ("Bradford" "PRTF" nil "BRDF")
      ("Bradstreet" "PRTS" nil "BRDS")
      ("Braham" "PRHM" nil "BRHM")
      ("Brailsford" "PRLS" nil "BRLS")
      ("Brainard" "PRNR" nil "BRNR")
      ("Brandish" "PRNT" nil "BRND")
      ("Braun" "PRN" nil "BRN")
      ("Brecc" "PRK" nil "BRK")
      ("Brent" "PRNT" nil "BRNT")
      ("Brenton" "PRNT" nil "BRNT")
      ("Briggs" "PRKS" nil "BRKS")
      ("Brigham" "PRM" nil "BRM")
      ("Brobst" "PRPS" nil "BRBS")
      ("Brome" "PRM" nil "BRM")
      ("Bronson" "PRNS" nil "BRNS")
      ("Brooks" "PRKS" nil "BRKS")
      ("Brouillard" "PRLR" nil "BRLR")
      ("Brown" "PRN" nil "BRN")
      ("Browne" "PRN" nil "BRN")
      ("Brownell" "PRNL" nil "BRNL")
      ("Bruley" "PRL" nil "BRL")
      ("Bryant" "PRNT" nil "BRNT")
      ("Brzozowski" "PRSS" "PRTS" "BRSS" "BRTS")
      ("Buide" "PT" nil "BD")
      ("Bulmer" "PLMR" nil "BLMR")
      ("Bunker" "PNKR" nil "BNKR")
      ("Burden" "PRTN" nil "BRDN")
      ("Burge" "PRJ" "PRK" "BRJ" "BRK")
      ("Burgoyne" "PRKN" nil "BRKN")
      ("Burke" "PRK" nil "BRK")
      ("Burnett" "PRNT" nil "BRNT")
      ("Burpee" "PRP" nil "BRP")
      ("Bursley" "PRSL" nil "BRSL")
      ("Burton" "PRTN" nil "BRTN")
      ("Bushnell" "PXNL" nil "BXNL")
      ("Buss" "PS" nil "BS")
      ("Buswell" "PSL" nil "BSL")
      ("Butler" "PTLR" nil "BTLR")
      ("Calkin" "KLKN" nil)
      ("Canada" "KNT" nil "KND")
      ("Canmore" "KNMR" nil)
      ("Canney" "KN" nil)
      ("Capet" "KPT" nil)
      ("Card" "KRT" nil "KRD")
      ("Carman" "KRMN" nil)
      ("Carpenter" "KRPN" nil)
      ("Cartwright" "KRTR" nil)
      ("Casey" "KS" nil)
      ("Catterfield" "KTRF" nil)
      ("Ceeley" "SL" nil)
      ("Chambers" "XMPR" nil "XMBR")
      ("Champion" "XMPN" nil)
      ("Chapman" "XPMN" nil)
      ("Chase" "XS" nil)
      ("Cheney" "XN" nil)
      ("Chetwynd" "XTNT" nil "XTND")
      ("Chevalier" "XFL" "XFLR" "XVL" "XVLR")
      ("Chillingsworth" "XLNK" nil)
      ("Christie" "KRST" nil)
      ("Chubbuck" "XPK" nil "XBK")
      ("Church" "XRX" "XRK")
      ("Clark" "KLRK" nil)
      ("Clarke" "KLRK" nil)
      ("Cleare" "KLR" nil)
      ("Clement" "KLMN" nil)
      ("Clerke" "KLRK" nil)
      ("Clibben" "KLPN" nil "KLBN")
      ("Clifford" "KLFR" nil)
      ("Clivedon" "KLFT" nil "KLVD")
      ("Close" "KLS" nil)
      ("Clothilde" "KL0L" "KLTL")
      ("Cobb" "KP" nil "KB")
      ("Coburn" "KPRN" nil "KBRN")
      ("Coburne" "KPRN" nil "KBRN")
      ("Cocke" "KK" nil)
      ("Coffin" "KFN" nil)
      ("Coffyn" "KFN" nil)
      ("Colborne" "KLPR" nil "KLBR")
      ("Colby" "KLP" nil "KLB")
      ("Cole" "KL" nil)
      ("Coleman" "KLMN" nil)
      ("Collier" "KL" "KLR")
      ("Compton" "KMPT" nil)
      ("Cone" "KN" nil)
      ("Cook" "KK" nil)
      ("Cooke" "KK" nil)
      ("Cooper" "KPR" nil)
      ("Copperthwaite" "KPR0" "KPRT")
      ("Corbet" "KRPT" nil "KRBT")
      ("Corell" "KRL" nil)
      ("Corey" "KR" nil)
      ("Corlies" "KRLS" nil)
      ("Corneliszen" "KRNL" nil)
      ("Cornelius" "KRNL" nil)
      ("Cornwallis" "KRNL" nil)
      ("Cosgrove" "KSKR" nil)
      ("Count of Brionne" "KNTF" nil)
      ("Covill" "KFL" nil "KVL")
      ("Cowperthwaite" "KPR0" "KPRT")
      ("Cowperwaite" "KPRT" nil)
      ("Crane" "KRN" nil)
      ("Creagmile" "KRKM" nil)
      ("Crew" "KR" "KRF")
      ("Crispin" "KRSP" nil)
      ("Crocker" "KRKR" nil)
      ("Crockett" "KRKT" nil)
      ("Crosby" "KRSP" nil "KRSB")
      ("Crump" "KRMP" nil)
      ("Cunningham" "KNNK" nil)
      ("Curtis" "KRTS" nil)
      ("Cutha" "K0" "KT")
      ("Cutter" "KTR" nil)
      ("D'Aubigny" "TPN" "TPKN" "DBN" "DBKN")
      ("Dabinott" "TPNT" nil "DBNT")
      ("Dacre" "TKR" nil "DKR")
      ("Daggett" "TKT" nil "DKT")
      ("Danvers" "TNFR" nil "DNVR")
      ("Darcy" "TRS" nil "DRS")
      ("Davis" "TFS" nil "DVS")
      ("Dawn" "TN" nil "DN")
      ("Dawson" "TSN" nil "DSN")
      ("Day" "T" nil "D")
      ("Daye" "T" nil "D")
      ("DeGrenier" "TKRN" nil "DKRN")
      ("Dean" "TN" nil "DN")
      ("Deekindaugh" "TKNT" nil "DKND")
      ("Dennis" "TNS" nil "DNS")
      ("Denny" "TN" nil "DN")
      ("Denton" "TNTN" nil "DNTN")
      ("Desborough" "TSPR" nil "DSBR")
      ("Despenser" "TSPN" nil "DSPN")
      ("Deverill" "TFRL" nil "DVRL")
      ("Devine" "TFN" nil "DVN")
      ("Dexter" "TKST" nil "DKST")
      ("Dillaway" "TL" nil "DL")
      ("Dimmick" "TMK" nil "DMK")
      ("Dinan" "TNN" nil "DNN")
      ("Dix" "TKS" nil "DKS")
      ("Doggett" "TKT" nil "DKT")
      ("Donahue" "TNH" nil "DNH")
      ("Dorfman" "TRFM" nil "DRFM")
      ("Dorris" "TRS" nil "DRS")
      ("Dow" "T" "TF" "D" "DF")
      ("Downey" "TN" nil "DN")
      ("Downing" "TNNK" nil "DNNK")
      ("Dowsett" "TST" nil "DST")
      ("Duck" "TK" nil "DK")
      ("Dudley" "TTL" nil "DDL")
      ("Duffy" "TF" nil "DF")
      ("Dunn" "TN" nil "DN")
      ("Dunsterville" "TNST" nil "DNST")
      ("Durrant" "TRNT" nil "DRNT")
      ("Durrin" "TRN" nil "DRN")
      ("Dustin" "TSTN" nil "DSTN")
      ("Duston" "TSTN" nil "DSTN")
      ("Eames" "AMS" nil)
      ("Early" "ARL" nil)
      ("Easty" "AST" nil)
      ("Ebbett" "APT" nil "ABT")
      ("Eberbach" "APRP" nil "ABRB")
      ("Eberhard" "APRR" nil "ABRR")
      ("Eddy" "AT" nil "AD")
      ("Edenden" "ATNT" nil "ADND")
      ("Edwards" "ATRT" nil "ADRD")
      ("Eglinton" "AKLN" "ALNT")
      ("Eliot" "ALT" nil)
      ("Elizabeth" "ALSP" nil "ALZB")
      ("Ellis" "ALS" nil)
      ("Ellison" "ALSN" nil)
      ("Ellot" "ALT" nil)
      ("Elny" "ALN" nil)
      ("Elsner" "ALSN" nil)
      ("Emerson" "AMRS" nil)
      ("Empson" "AMPS" nil)
      ("Est" "AST" nil)
      ("Estabrook" "ASTP" nil "ASTB")
      ("Estes" "ASTS" nil)
      ("Estey" "AST" nil)
      ("Evans" "AFNS" nil "AVNS")
      ("Fallowell" "FLL" nil)
      ("Farnsworth" "FRNS" nil)
      ("Feake" "FK" nil)
      ("Feke" "FK" nil)
      ("Fellows" "FLS" nil)
      ("Fettiplace" "FTPL" nil)
      ("Finney" "FN" nil)
      ("Fischer" "FXR" "FSKR")
      ("Fisher" "FXR" nil)
      ("Fisk" "FSK" nil)
      ("Fiske" "FSK" nil)
      ("Fletcher" "FLXR" nil)
      ("Folger" "FLKR" "FLJR")
      ("Foliot" "FLT" nil)
      ("Folyot" "FLT" nil)
      ("Fones" "FNS" nil)
      ("Fordham" "FRTM" nil "FRDM")
      ("Forstner" "FRST" nil)
      ("Fosten" "FSTN" nil)
      ("Foster" "FSTR" nil)
      ("Foulke" "FLK" nil)
      ("Fowler" "FLR" nil)
      ("Foxwell" "FKSL" nil)
      ("Fraley" "FRL" nil)
      ("Franceys" "FRNS" nil)
      ("Franke" "FRNK" nil)
      ("Frascella" "FRSL" nil)
      ("Frazer" "FRSR" nil "FRZR")
      ("Fredd" "FRT" nil "FRD")
      ("Freeman" "FRMN" nil)
      ("French" "FRNX" "FRNK")
      ("Freville" "FRFL" nil "FRVL")
      ("Frey" "FR" nil)
      ("Frick" "FRK" nil)
      ("Frier" "FR" "FRR")
      ("Froe" "FR" nil)
      ("Frorer" "FRRR" nil)
      ("Frost" "FRST" nil)
      ("Frothingham" "FR0N" "FRTN")
      ("Fry" "FR" nil)
      ("Gaffney" "KFN" nil)
      ("Gage" "KJ" "KK")
      ("Gallion" "KLN" nil)
      ("Gallishan" "KLXN" nil)
      ("Gamble" "KMPL" nil "KMBL")
      ("Garbrand" "KRPR" nil "KRBR")
      ("Gardner" "KRTN" nil "KRDN")
      ("Garrett" "KRT" nil)
      ("Gassner" "KSNR" nil)
      ("Gater" "KTR" nil)
      ("Gaunt" "KNT" nil)
      ("Gayer" "KR" nil)
      ("Gerken" "KRKN" "JRKN")
      ("Gerritsen" "KRTS" "JRTS")
      ("Gibbs" "KPS" "JPS" "KBS" "JBS")
      ("Giffard" "JFRT" "KFRT" "JFRD" "KFRD")
      ("Gilbert" "KLPR" "JLPR" "KLBR" "JLBR")
      ("Gill" "KL" "JL")
      ("Gilman" "KLMN" "JLMN")
      ("Glass" "KLS" nil)
      ("Goddard/Gifford" "KTRT" nil "KDRD")
      ("Godfrey" "KTFR" nil "KDFR")
      ("Godwin" "KTN" nil "KDN")
      ("Goodale" "KTL" nil "KDL")
      ("Goodnow" "KTN" "KTNF" "KDN" "KDNF")
      ("Gorham" "KRM" nil)
      ("Goseline" "KSLN" nil)
      ("Gott" "KT" nil)
      ("Gould" "KLT" nil "KLD")
      ("Grafton" "KRFT" nil)
      ("Grant" "KRNT" nil)
      ("Gray" "KR" nil)
      ("Green" "KRN" nil)
      ("Griffin" "KRFN" nil)
      ("Grill" "KRL" nil)
      ("Grim" "KRM" nil)
      ("Grisgonelle" "KRSK" nil)
      ("Gross" "KRS" nil)
      ("Guba" "KP" nil "KB")
      ("Gybbes" "KPS" "JPS" "KBS" "JBS")
      ("Haburne" "HPRN" nil "HBRN")
      ("Hackburne" "HKPR" nil "HKBR")
      ("Haddon" "HTN" nil "HDN")
      ("Haines" "HNS" nil)
      ("Hale" "HL" nil)
      ("Hall" "HL" nil)
      ("Hallet" "HLT" nil)
      ("Hallock" "HLK" nil)
      ("Halstead" "HLST" nil)
      ("Hammond" "HMNT" nil "HMND")
      ("Hance" "HNS" nil)
      ("Handy" "HNT" nil "HND")
      ("Hanson" "HNSN" nil)
      ("Harasek" "HRSK" nil)
      ("Harcourt" "HRKR" nil)
      ("Hardy" "HRT" nil "HRD")
      ("Harlock" "HRLK" nil)
      ("Harris" "HRS" nil)
      ("Hartley" "HRTL" nil)
      ("Harvey" "HRF" nil "HRV")
      ("Harvie" "HRF" nil "HRV")
      ("Harwood" "HRT" nil "HRD")
      ("Hathaway" "H0" "HT")
      ("Haukeness" "HKNS" nil)
      ("Hawkes" "HKS" nil)
      ("Hawkhurst" "HKRS" nil)
      ("Hawkins" "HKNS" nil)
      ("Hawley" "HL" nil)
      ("Heald" "HLT" nil "HLD")
      ("Helsdon" "HLST" nil "HLSD")
      ("Hemenway" "HMN" nil)
      ("Hemmenway" "HMN" nil)
      ("Henck" "HNK" nil)
      ("Henderson" "HNTR" nil "HNDR")
      ("Hendricks" "HNTR" nil "HNDR")
      ("Hersey" "HRS" nil)
      ("Hewes" "HS" nil)
      ("Heyman" "HMN" nil)
      ("Hicks" "HKS" nil)
      ("Hidden" "HTN" nil "HDN")
      ("Higgs" "HKS" nil)
      ("Hill" "HL" nil)
      ("Hills" "HLS" nil)
      ("Hinckley" "HNKL" nil)
      ("Hipwell" "HPL" nil)
      ("Hobart" "HPRT" nil "HBRT")
      ("Hoben" "HPN" nil "HBN")
      ("Hoffmann" "HFMN" nil)
      ("Hogan" "HKN" nil)
      ("Holmes" "HLMS" nil)
      ("Hoo" "H" nil)
      ("Hooker" "HKR" nil)
      ("Hopcott" "HPKT" nil)
      ("Hopkins" "HPKN" nil)
      ("Hopkinson" "HPKN" nil)
      ("Hornsey" "HRNS" nil)
      ("Houckgeest" "HKJS" "HKKS")
      ("Hough" "H" nil)
      ("Houstin" "HSTN" nil)
      ("How" "H" "HF")
      ("Howe" "H" nil)
      ("Howland" "HLNT" nil "HLND")
      ("Hubner" "HPNR" nil "HBNR")
      ("Hudnut" "HTNT" nil "HDNT")
      ("Hughes" "HS" nil)
      ("Hull" "HL" nil)
      ("Hulme" "HLM" nil)
      ("Hume" "HM" nil)
      ("Hundertumark" "HNTR" nil "HNDR")
      ("Hundley" "HNTL" nil "HNDL")
      ("Hungerford" "HNKR" "HNJR")
      ("Hunt" "HNT" nil)
      ("Hurst" "HRST" nil)
      ("Husbands" "HSPN" nil "HSBN")
      ("Hussey" "HS" nil)
      ("Husted" "HSTT" nil "HSTD")
      ("Hutchins" "HXNS" nil)
      ("Hutchinson" "HXNS" nil)
      ("Huttinger" "HTNK" "HTNJ")
      ("Huybertsen" "HPRT" nil "HBRT")
      ("Iddenden" "ATNT" nil "ADND")
      ("Ingraham" "ANKR" nil)
      ("Ives" "AFS" nil "AVS")
      ("Jackson" "JKSN" "AKSN")
      ("Jacob" "JKP" "AKP" "JKB" "AKB")
      ("Jans" "JNS" "ANS")
      ("Jenkins" "JNKN" "ANKN")
      ("Jewett" "JT" "AT")
      ("Jewitt" "JT" "AT")
      ("Johnson" "JNSN" "ANSN")
      ("Jones" "JNS" "ANS")
      ("Josephine" "JSFN" "HSFN")
      ("Judd" "JT" "AT" "JD" "AD")
      ("June" "JN" "AN")
      ("Kamarowska" "KMRS" nil)
      ("Kay" "K" nil)
      ("Kelley" "KL" nil)
      ("Kelly" "KL" nil)
      ("Keymber" "KMPR" nil "KMBR")
      ("Keynes" "KNS" nil)
      ("Kilham" "KLM" nil)
      ("Kim" "KM" nil)
      ("Kimball" "KMPL" nil "KMBL")
      ("King" "KNK" nil)
      ("Kinsey" "KNS" nil)
      ("Kirk" "KRK" nil)
      ("Kirton" "KRTN" nil)
      ("Kistler" "KSTL" nil)
      ("Kitchen" "KXN" nil)
      ("Kitson" "KTSN" nil)
      ("Klett" "KLT" nil)
      ("Kline" "KLN" nil)
      ("Knapp" "NP" nil)
      ("Knight" "NT" nil)
      ("Knote" "NT" nil)
      ("Knott" "NT" nil)
      ("Knox" "NKS" nil)
      ("Koeller" "KLR" nil)
      ("La Pointe" "LPNT" nil)
      ("LaPlante" "LPLN" nil)
      ("Laimbeer" "LMPR" nil "LMBR")
      ("Lamb" "LMP" nil "LMB")
      ("Lambertson" "LMPR" nil "LMBR")
      ("Lancto" "LNKT" nil)
      ("Landry" "LNTR" nil "LNDR")
      ("Lane" "LN" nil)
      ("Langendyck" "LNJN" "LNKN")
      ("Langer" "LNKR" "LNJR")
      ("Langford" "LNKF" nil)
      ("Lantersee" "LNTR" nil)
      ("Laquer" "LKR" nil)
      ("Larkin" "LRKN" nil)
      ("Latham" "LTM" nil)
      ("Lathrop" "L0RP" "LTRP")
      ("Lauter" "LTR" nil)
      ("Lawrence" "LRNS" nil)
      ("Leach" "LK" nil)
      ("Leager" "LKR" "LJR")
      ("Learned" "LRNT" nil "LRND")
      ("Leavitt" "LFT" nil "LVT")
      ("Lee" "L" nil)
      ("Leete" "LT" nil)
      ("Leggett" "LKT" nil)
      ("Leland" "LLNT" nil "LLND")
      ("Leonard" "LNRT" nil "LNRD")
      ("Lester" "LSTR" nil)
      ("Lestrange" "LSTR" nil)
      ("Lethem" "L0M" "LTM")
      ("Levine" "LFN" nil "LVN")
      ("Lewes" "LS" nil)
      ("Lewis" "LS" nil)
      ("Lincoln" "LNKL" nil)
      ("Lindsey" "LNTS" nil "LNDS")
      ("Linher" "LNR" nil)
      ("Lippet" "LPT" nil)
      ("Lippincott" "LPNK" nil)
      ("Lockwood" "LKT" nil "LKD")
      ("Loines" "LNS" nil)
      ("Lombard" "LMPR" nil "LMBR")
      ("Long" "LNK" nil)
      ("Longespee" "LNJS" "LNKS")
      ("Look" "LK" nil)
      ("Lounsberry" "LNSP" nil "LNSB")
      ("Lounsbury" "LNSP" nil "LNSB")
      ("Louthe" "L0" "LT")
      ("Loveyne" "LFN" nil "LVN")
      ("Lowe" "L" nil)
      ("Ludlam" "LTLM" nil "LDLM")
      ("Lumbard" "LMPR" nil "LMBR")
      ("Lund" "LNT" nil "LND")
      ("Luno" "LN" nil)
      ("Lutz" "LTS" nil "LTZ")
      ("Lydia" "LT" nil "LD")
      ("Lynne" "LN" nil)
      ("Lyon" "LN" nil)
      ("MacAlpin" "MKLP" nil)
      ("MacBricc" "MKPR" nil "MKBR")
      ("MacCrinan" "MKRN" nil)
      ("MacKenneth" "MKN0" "MKNT")
      ("MacMael nam Bo" "MKML" nil)
      ("MacMurchada" "MKMR" nil)
      ("Macomber" "MKMP" nil "MKMB")
      ("Macy" "MS" nil)
      ("Magnus" "MNS" "MKNS")
      ("Mahien" "MHN" nil)
      ("Malmains" "MLMN" nil)
      ("Malory" "MLR" nil)
      ("Mancinelli" "MNSN" nil)
      ("Mancini" "MNSN" nil)
      ("Mann" "MN" nil)
      ("Manning" "MNNK" nil)
      ("Manter" "MNTR" nil)
      ("Marion" "MRN" nil)
      ("Marley" "MRL" nil)
      ("Marmion" "MRMN" nil)
      ("Marquart" "MRKR" nil)
      ("Marsh" "MRX" nil)
      ("Marshal" "MRXL" nil)
      ("Marshall" "MRXL" nil)
      ("Martel" "MRTL" nil)
      ("Martha" "MR0" "MRT")
      ("Martin" "MRTN" nil)
      ("Marturano" "MRTR" nil)
      ("Marvin" "MRFN" nil "MRVN")
      ("Mary" "MR" nil)
      ("Mason" "MSN" nil)
      ("Maxwell" "MKSL" nil)
      ("Mayhew" "MH" "MHF")
      ("McAllaster" "MKLS" nil)
      ("McAllister" "MKLS" nil)
      ("McConnell" "MKNL" nil)
      ("McFarland" "MKFR" nil)
      ("McIlroy" "MSLR" nil)
      ("McNair" "MKNR" nil)
      ("McNair-Landry" "MKNR" nil)
      ("McRaven" "MKRF" nil "MKRV")
      ("Mead" "MT" nil "MD")
      ("Meade" "MT" nil "MD")
      ("Meck" "MK" nil)
      ("Melton" "MLTN" nil)
      ("Mendenhall" "MNTN" nil "MNDN")
      ("Mering" "MRNK" nil)
      ("Merrick" "MRK" nil)
      ("Merry" "MR" nil)
      ("Mighill" "ML" nil)
      ("Miller" "MLR" nil)
      ("Milton" "MLTN" nil)
      ("Mohun" "MHN" nil)
      ("Montague" "MNTK" nil)
      ("Montboucher" "MNTP" nil "MNTB")
      ("Moore" "MR" nil)
      ("Morrel" "MRL" nil)
      ("Morrill" "MRL" nil)
      ("Morris" "MRS" nil)
      ("Morton" "MRTN" nil)
      ("Moton" "MTN" nil)
      ("Muir" "MR" nil)
      ("Mulferd" "MLFR" nil)
      ("Mullins" "MLNS" nil)
      ("Mulso" "MLS" nil)
      ("Munger" "MNKR" "MNJR")
      ("Munt" "MNT" nil)
      ("Murchad" "MRXT" "MRKT" "MRXD" "MRKD")
      ("Murdock" "MRTK" nil "MRDK")
      ("Murray" "MR" nil)
      ("Muskett" "MSKT" nil)
      ("Myers" "MRS" nil)
      ("Myrick" "MRK" nil)
      ("NORRIS" "NRS" nil)
      ("Nayle" "NL" nil)
      ("Newcomb" "NKMP" nil "NKMB")
      ("Newcombe" "NKMP" nil "NKMB")
      ("Newkirk" "NKRK" nil)
      ("Newton" "NTN" nil)
      ("Niles" "NLS" nil)
      ("Noble" "NPL" nil "NBL")
      ("Noel" "NL" nil)
      ("Northend" "NR0N" "NRTN")
      ("Norton" "NRTN" nil)
      ("Nutter" "NTR" nil)
      ("Odding" "ATNK" nil "ADNK")
      ("Odenbaugh" "ATNP" nil "ADNB")
      ("Ogborn" "AKPR" nil "AKBR")
      ("Oppenheimer" "APNM" nil)
      ("Otis" "ATS" nil)
      ("Oviatt" "AFT" nil "AVT")
      ("Prust" "PRST" nil)
      ("Paddock" "PTK" nil "PDK")
      ("Page" "PJ" "PK")
      ("Paine" "PN" nil)
      ("Paist" "PST" nil)
      ("Palmer" "PLMR" nil)
      ("Park" "PRK" nil)
      ("Parker" "PRKR" nil)
      ("Parkhurst" "PRKR" nil)
      ("Parrat" "PRT" nil)
      ("Parsons" "PRSN" nil)
      ("Partridge" "PRTR" nil)
      ("Pashley" "PXL" nil)
      ("Pasley" "PSL" nil)
      ("Patrick" "PTRK" nil)
      ("Pattee" "PT" nil)
      ("Patten" "PTN" nil)
      ("Pawley" "PL" nil)
      ("Payne" "PN" nil)
      ("Peabody" "PPT" nil "PBD")
      ("Peake" "PK" nil)
      ("Pearson" "PRSN" nil)
      ("Peat" "PT" nil)
      ("Pedersen" "PTRS" nil "PDRS")
      ("Percy" "PRS" nil)
      ("Perkins" "PRKN" nil)
      ("Perrine" "PRN" nil)
      ("Perry" "PR" nil)
      ("Peson" "PSN" nil)
      ("Peterson" "PTRS" nil)
      ("Peyton" "PTN" nil)
      ("Phinney" "FN" nil)
      ("Pickard" "PKRT" nil "PKRD")
      ("Pierce" "PRS" nil)
      ("Pierrepont" "PRPN" nil)
      ("Pike" "PK" nil)
      ("Pinkham" "PNKM" nil)
      ("Pitman" "PTMN" nil)
      ("Pitt" "PT" nil)
      ("Pitts" "PTS" nil)
      ("Plantagenet" "PLNT" nil)
      ("Platt" "PLT" nil)
      ("Platts" "PLTS" nil)
      ("Pleis" "PLS" nil)
      ("Pleiss" "PLS" nil)
      ("Plisko" "PLSK" nil)
      ("Pliskovitch" "PLSK" nil)
      ("Plum" "PLM" nil)
      ("Plume" "PLM" nil)
      ("Poitou" "PT" nil)
      ("Pomeroy" "PMR" nil)
      ("Poretiers" "PRTR" nil)
      ("Pote" "PT" nil)
      ("Potter" "PTR" nil)
      ("Potts" "PTS" nil)
      ("Powell" "PL" nil)
      ("Pratt" "PRT" nil)
      ("Presbury" "PRSP" nil "PRSB")
      ("Priest" "PRST" nil)
      ("Prindle" "PRNT" nil "PRND")
      ("Prior" "PRR" nil)
      ("Profumo" "PRFM" nil)
      ("Purdy" "PRT" nil "PRD")
      ("Purefoy" "PRF" nil)
      ("Pury" "PR" nil)
      ("Quinter" "KNTR" nil)
      ("Rachel" "RXL" "RKL")
      ("Rand" "RNT" nil "RND")
      ("Rankin" "RNKN" nil)
      ("Ravenscroft" "RFNS" nil "RVNS")
      ("Raynsford" "RNSF" nil)
      ("Reakirt" "RKRT" nil)
      ("Reaves" "RFS" nil "RVS")
      ("Reeves" "RFS" nil "RVS")
      ("Reichert" "RXRT" "RKRT")
      ("Remmele" "RML" nil)
      ("Reynolds" "RNLT" nil "RNLD")
      ("Rhodes" "RTS" nil "RDS")
      ("Richards" "RXRT" "RKRT" "RXRD" "RKRD")
      ("Richardson" "RXRT" "RKRT" "RXRD" "RKRD")
      ("Ring" "RNK" nil)
      ("Roberts" "RPRT" nil "RBRT")
      ("Robertson" "RPRT" nil "RBRT")
      ("Robson" "RPSN" nil "RBSN")
      ("Rodie" "RT" nil "RD")
      ("Rody" "RT" nil "RD")
      ("Rogers" "RKRS" "RJRS")
      ("Ross" "RS" nil)
      ("Rosslevin" "RSLF" nil "RSLV")
      ("Rowland" "RLNT" nil "RLND")
      ("Ruehl" "RL" nil)
      ("Russell" "RSL" nil)
      ("Ruth" "R0" "RT")
      ("Ryan" "RN" nil)
      ("Rysse" "RS" nil)
      ("Sadler" "STLR" nil "SDLR")
      ("Salmon" "SLMN" nil)
      ("Salter" "SLTR" nil)
      ("Salvatore" "SLFT" nil "SLVT")
      ("Sanders" "SNTR" nil "SNDR")
      ("Sands" "SNTS" nil "SNDS")
      ("Sanford" "SNFR" nil)
      ("Sanger" "SNKR" "SNJR")
      ("Sargent" "SRJN" "SRKN")
      ("Saunders" "SNTR" nil "SNDR")
      ("Schilling" "XLNK" nil)
      ("Schlegel" "XLKL" "SLKL")
      ("Scott" "SKT" nil)
      ("Sears" "SRS" nil)
      ("Segersall" "SJRS" "SKRS")
      ("Senecal" "SNKL" nil)
      ("Sergeaux" "SRJ" "SRK")
      ("Severance" "SFRN" nil "SVRN")
      ("Sharp" "XRP" nil)
      ("Sharpe" "XRP" nil)
      ("Sharply" "XRPL" nil)
      ("Shatswell" "XTSL" nil)
      ("Shattack" "XTK" nil)
      ("Shattock" "XTK" nil)
      ("Shattuck" "XTK" nil)
      ("Shaw" "X" "XF")
      ("Sheldon" "XLTN" nil "XLDN")
      ("Sherman" "XRMN" nil)
      ("Shinn" "XN" nil)
      ("Shirford" "XRFR" nil)
      ("Shirley" "XRL" nil)
      ("Shively" "XFL" nil "XVL")
      ("Shoemaker" "XMKR" nil)
      ("Short" "XRT" nil)
      ("Shotwell" "XTL" nil)
      ("Shute" "XT" nil)
      ("Sibley" "SPL" nil "SBL")
      ("Silver" "SLFR" nil "SLVR")
      ("Simes" "SMS" nil)
      ("Sinken" "SNKN" nil)
      ("Sinn" "SN" nil)
      ("Skelton" "SKLT" nil)
      ("Skiffe" "SKF" nil)
      ("Skotkonung" "SKTK" nil)
      ("Slade" "SLT" "XLT" "SLD" "XLD")
      ("Slye" "SL" "XL")
      ("Smedley" "SMTL" "XMTL" "SMDL" "XMDL")
      ("Smith" "SM0" "XMT")
      ("Snow" "SN" "XNF")
      ("Soole" "SL" nil)
      ("Soule" "SL" nil)
      ("Southworth" "S0R0" "STRT")
      ("Sowles" "SLS" nil)
      ("Spalding" "SPLT" nil "SPLD")
      ("Spark" "SPRK" nil)
      ("Spencer" "SPNS" nil)
      ("Sperry" "SPR" nil)
      ("Spofford" "SPFR" nil)
      ("Spooner" "SPNR" nil)
      ("Sprague" "SPRK" nil)
      ("Springer" "SPRN" nil)
      ("St. Clair" "STKL" nil)
      ("St. Claire" "STKL" nil)
      ("St. Leger" "STLJ" "STLK")
      ("St. Omer" "STMR" nil)
      ("Stafferton" "STFR" nil)
      ("Stafford" "STFR" nil)
      ("Stalham" "STLM" nil)
      ("Stanford" "STNF" nil)
      ("Stanton" "STNT" nil)
      ("Star" "STR" nil)
      ("Starbuck" "STRP" nil "STRB")
      ("Starkey" "STRK" nil)
      ("Starkweather" "STRK" nil)
      ("Stearns" "STRN" nil)
      ("Stebbins" "STPN" nil "STBN")
      ("Steele" "STL" nil)
      ("Stephenson" "STFN" nil)
      ("Stevens" "STFN" nil "STVN")
      ("Stoddard" "STTR" nil "STDR")
      ("Stodder" "STTR" nil "STDR")
      ("Stone" "STN" nil)
      ("Storey" "STR" nil)
      ("Storrada" "STRT" nil "STRD")
      ("Story" "STR" nil)
      ("Stoughton" "STFT" nil)
      ("Stout" "STT" nil)
      ("Stow" "ST" "STF")
      ("Strong" "STRN" nil)
      ("Strutt" "STRT" nil)
      ("Stryker" "STRK" nil)
      ("Stuckeley" "STKL" nil)
      ("Sturges" "STRJ" "STRK")
      ("Sturgess" "STRJ" "STRK")
      ("Sturgis" "STRJ" "STRK")
      ("Suevain" "SFN" nil "SVN")
      ("Sulyard" "SLRT" nil "SLRD")
      ("Sutton" "STN" nil)
      ("Swain" "SN" "XN")
      ("Swayne" "SN" "XN")
      ("Swayze" "SS" "XTS")
      ("Swift" "SFT" "XFT")
      ("Taber" "TPR" nil "TBR")
      ("Talcott" "TLKT" nil)
      ("Tarne" "TRN" nil)
      ("Tatum" "TTM" nil)
      ("Taverner" "TFRN" nil "TVRN")
      ("Taylor" "TLR" nil)
      ("Tenney" "TN" nil)
      ("Thayer" "0R" "TR")
      ("Thember" "0MPR" "TMPR" "0MBR" "TMBR")
      ("Thomas" "TMS" nil)
      ("Thompson" "TMPS" nil)
      ("Thorne" "0RN" "TRN")
      ("Thornycraft" "0RNK" "TRNK")
      ("Threlkeld" "0RLK" "TRLK")
      ("Throckmorton" "0RKM" "TRKM")
      ("Thwaits" "0TS" "TTS")
      ("Tibbetts" "TPTS" nil "TBTS")
      ("Tidd" "TT" nil "TD")
      ("Tierney" "TRN" nil)
      ("Tilley" "TL" nil)
      ("Tillieres" "TLRS" nil)
      ("Tilly" "TL" nil)
      ("Tisdale" "TSTL" nil "TSDL")
      ("Titus" "TTS" nil)
      ("Tobey" "TP" nil "TB")
      ("Tooker" "TKR" nil)
      ("Towle" "TL" nil)
      ("Towne" "TN" nil)
      ("Townsend" "TNSN" nil)
      ("Treadway" "TRT" nil "TRD")
      ("Trelawney" "TRLN" nil)
      ("Trinder" "TRNT" nil "TRND")
      ("Tripp" "TRP" nil)
      ("Trippe" "TRP" nil)
      ("Trott" "TRT" nil)
      ("True" "TR" nil)
      ("Trussebut" "TRSP" nil "TRSB")
      ("Tucker" "TKR" nil)
      ("Turgeon" "TRJN" "TRKN")
      ("Turner" "TRNR" nil)
      ("Tuttle" "TTL" nil)
      ("Tyler" "TLR" nil)
      ("Tylle" "TL" nil)
      ("Tyrrel" "TRL" nil)
      ("Ua Tuathail" "AT0L" "ATTL")
      ("Ulrich" "ALRX" "ALRK")
      ("Underhill" "ANTR" nil "ANDR")
      ("Underwood" "ANTR" nil "ANDR")
      ("Unknown" "ANKN" nil)
      ("Valentine" "FLNT" nil "VLNT")
      ("Van Egmond" "FNKM" nil "VNKM")
      ("Van der Beek" "FNTR" nil "VNDR")
      ("Vaughan" "FKN" nil "VKN")
      ("Vermenlen" "FRMN" nil "VRMN")
      ("Vincent" "FNSN" nil "VNSN")
      ("Volentine" "FLNT" nil "VLNT")
      ("Wagner" "AKNR" "FKNR")
      ("Waite" "AT" "FT")
      ("Walker" "ALKR" "FLKR")
      ("Walter" "ALTR" "FLTR")
      ("Wandell" "ANTL" "FNTL" "ANDL" "FNDL")
      ("Wandesford" "ANTS" "FNTS" "ANDS" "FNDS")
      ("Warbleton" "ARPL" "FRPL" "ARBL" "FRBL")
      ("Ward" "ART" "FRT" "ARD" "FRD")
      ("Warde" "ART" "FRT"  "ARD" "FRD")
      ("Ware" "AR" "FR")
      ("Wareham" "ARHM" "FRHM")
      ("Warner" "ARNR" "FRNR")
      ("Warren" "ARN" "FRN")
      ("Washburne" "AXPR" "FXPR" "AXBR" "FXBR")
      ("Waterbury" "ATRP" "FTRP" "ATRB" "FTRB")
      ("Watson" "ATSN" "FTSN")
      ("WatsonEllithorpe" "ATSN" "FTSN")
      ("Watts" "ATS" "FTS")
      ("Wayne" "AN" "FN")
      ("Webb" "AP" "FP" "AB" "FB")
      ("Weber" "APR" "FPR" "ABR" "FBR")
      ("Webster" "APST" "FPST" "ABST" "FBST")
      ("Weed" "AT" "FT" "AD" "FD")
      ("Weeks" "AKS" "FKS")
      ("Wells" "ALS" "FLS")
      ("Wenzell" "ANSL" "FNTS")
      ("West" "AST" "FST")
      ("Westbury" "ASTP" "FSTP" "ASTB" "FSTB")
      ("Whatlocke" "ATLK" nil)
      ("Wheeler" "ALR" nil)
      ("Whiston" "ASTN" nil)
      ("White" "AT" nil)
      ("Whitman" "ATMN" nil)
      ("Whiton" "ATN" nil)
      ("Whitson" "ATSN" nil)
      ("Wickes" "AKS" "FKS")
      ("Wilbur" "ALPR" "FLPR" "ALBR" "FLBR")
      ("Wilcotes" "ALKT" "FLKT")
      ("Wilkinson" "ALKN" "FLKN")
      ("Willets" "ALTS" "FLTS")
      ("Willett" "ALT" "FLT")
      ("Willey" "AL" "FL")
      ("Williams" "ALMS" "FLMS")
      ("Williston" "ALST" "FLST")
      ("Wilson" "ALSN" "FLSN")
      ("Wimes" "AMS" "FMS")
      ("Winch" "ANX" "FNK")
      ("Winegar" "ANKR" "FNKR")
      ("Wing" "ANK" "FNK")
      ("Winsley" "ANSL" "FNSL")
      ("Winslow" "ANSL" "FNSL")
      ("Winthrop" "AN0R" "FNTR")
      ("Wise" "AS" "FS")
      ("Wood" "AT" "FT" "AD" "FD")
      ("Woodbridge" "ATPR" "FTPR" "ADBR" "FDBR")
      ("Woodward" "ATRT" "FTRT" "ADRD" "FDRD")
      ("Wooley" "AL" "FL")
      ("Woolley" "AL" "FL")
      ("Worth" "AR0" "FRT")
      ("Worthen" "AR0N" "FRTN")
      ("Worthley" "AR0L" "FRTL")
      ("Wright" "RT" nil)
      ("Wyer" "AR" "FR")
      ("Wyere" "AR" "FR")
      ("Wynkoop" "ANKP" "FNKP")
      ("Yarnall" "ARNL" nil)
      ("Yeoman" "AMN" nil)
      ("Yorke" "ARK" nil)
      ("Young" "ANK" nil)
      ("ab Wennonwen" "APNN" nil "ABNN")
      ("ap Llewellyn" "APLL" nil)
      ("ap Lorwerth" "APLR" nil)
      ("d'Angouleme" "TNKL" nil "DNKL")
      ("de Audeham" "TTHM" nil "DDHM")
      ("de Bavant" "TPFN" nil "DBVN")
      ("de Beauchamp" "TPXM" "TPKM" "DBXM" "DBKM")
      ("de Beaumont" "TPMN" nil "DBMN")
      ("de Bolbec" "TPLP" nil "DBLB")
      ("de Braiose" "TPRS" nil "DBRS")
      ("de Braose" "TPRS" nil "DBRS")
      ("de Briwere" "TPRR" nil "DBRR")
      ("de Cantelou" "TKNT" nil "DKNT")
      ("de Cherelton" "TXRL" "TKRL" "DXRL" "DKRL")
      ("de Cherleton" "TXRL" "TKRL"  "DXRL" "DKRL")
      ("de Clare" "TKLR" nil "DKLR")
      ("de Claremont" "TKLR" nil "DKLR")
      ("de Clifford" "TKLF" nil "DKLF")
      ("de Colville" "TKLF" nil "DKLV")
      ("de Courtenay" "TKRT" nil "DKRT")
      ("de Fauconberg" "TFKN" nil "DFKN")
      ("de Forest" "TFRS" nil "DFRS")
      ("de Gai" "TK" nil "DK")
      ("de Grey" "TKR" nil "DKR")
      ("de Guernons" "TKRN" nil "DKRN")
      ("de Haia" "T" nil "D")
      ("de Harcourt" "TRKR" nil "DRKR")
      ("de Hastings" "TSTN" nil "DSTN")
      ("de Hoke" "TK" nil "DK")
      ("de Hooch" "TK" nil "DK")
      ("de Hugelville" "TJLF" "TKLF" "DJLV" "DKLV")
      ("de Huntingdon" "TNTN" nil "DNTN")
      ("de Insula" "TNSL" nil "DNSL")
      ("de Keynes" "TKNS" nil "DKNS")
      ("de Lacy" "TLS" nil "DLS")
      ("de Lexington" "TLKS" nil "DLKS")
      ("de Lusignan" "TLSN" "TLSK" "DLSN" "DLSK")
      ("de Manvers" "TMNF" nil "DMNV")
      ("de Montagu" "TMNT" nil "DMNT")
      ("de Montault" "TMNT" nil "DMNT")
      ("de Montfort" "TMNT" nil "DMNT")
      ("de Mortimer" "TMRT" nil "DMRT")
      ("de Morville" "TMRF" nil "DMRV")
      ("de Morvois" "TMRF" nil "DMRV")
      ("de Neufmarche" "TNFM" nil "DNFM")
      ("de Odingsells" "TTNK" nil "DDNK")
      ("de Odyngsells" "TTNK" nil "DDNK")
      ("de Percy" "TPRS" nil "DPRS")
      ("de Pierrepont" "TPRP" nil "DPRP")
      ("de Plessetis" "TPLS" nil "DPLS")
      ("de Porhoet" "TPRT" nil "DPRT")
      ("de Prouz" "TPRS" nil "DPRZ")
      ("de Quincy" "TKNS" nil "DKNS")
      ("de Ripellis" "TRPL" nil "DRPL")
      ("de Ros" "TRS" nil "DRS")
      ("de Salisbury" "TSLS" nil "DSLS")
      ("de Sanford" "TSNF" nil "DSNF")
      ("de Somery" "TSMR" nil "DSMR")
      ("de St. Hilary" "TSTL" nil "DSTL")
      ("de St. Liz" "TSTL" nil "DSTL")
      ("de Sutton" "TSTN" nil "DSTN")
      ("de Toeni" "TTN" nil "DTN")
      ("de Tony" "TTN" nil "DTN")
      ("de Umfreville" "TMFR" nil "DMFR")
      ("de Valognes" "TFLN" "TFLK" "DVLN" "DVLK")
      ("de Vaux" "TF" nil "DV")
      ("de Vere" "TFR" nil "DVR")
      ("de Vermandois" "TFRM" nil "DVRM")
      ("de Vernon" "TFRN" nil "DVRN")
      ("de Vexin" "TFKS" nil "DVKS")
      ("de Vitre" "TFTR" nil "DVTR")
      ("de Wandesford" "TNTS" nil "DNDS")
      ("de Warenne" "TRN" nil "DRN")
      ("de Westbury" "TSTP" nil "DSTB")
      ("di Saluzzo" "TSLS" "TSLT" "DSLS" "DSLT")
      ("fitz Alan" "FTSL" nil "FTZL")
      ("fitz Geoffrey" "FTSJ" "FTSK" "FTZJ" "FTZK")
      ("fitz Herbert" "FTSR" nil "FTZR")
      ("fitz John" "FTSJ" nil "FTZJ")
      ("fitz Patrick" "FTSP" nil "FTZP")
      ("fitz Payn" "FTSP" nil "FTZP")
      ("fitz Piers" "FTSP" nil "FTZP")
      ("fitz Randolph" "FTSR" nil "FTZR")
      ("fitz Richard" "FTSR" nil "FTZR")
      ("fitz Robert" "FTSR" nil "FTZR")
      ("fitz Roy" "FTSR" nil "FTZR")
      ("fitz Scrob" "FTSS" nil "FTZS")
      ("fitz Walter" "FTSL" nil"FTZL")
      ("fitz Warin" "FTSR" nil "FTZR")
      ("fitz Williams" "FTSL" nil "FTZL")
      ("la Zouche" "LSX" "LSK" "LZX" "LZK")
      ("le Botiller" "LPTL" nil "LBTL")
      ("le Despenser" "LTSP" nil "LDSP")
      ("le deSpencer" "LTSP" nil "LDSP")
      ("of Allendale" "AFLN" nil "AFLN")
      ("of Angouleme" "AFNK" nil)
      ("of Anjou" "AFNJ" nil)
      ("of Aquitaine" "AFKT" nil)
      ("of Aumale" "AFML" nil)
      ("of Bavaria" "AFPF" nil "AFBV")
      ("of Boulogne" "AFPL" nil "AFBL")
      ("of Brittany" "AFPR" nil "AFBR")
      ("of Brittary" "AFPR" nil "AFBR")
      ("of Castile" "AFKS" nil)
      ("of Chester" "AFXS" "AFKS")
      ("of Clermont" "AFKL" nil)
      ("of Cologne" "AFKL" nil)
      ("of Dinan" "AFTN" nil "AFDN")
      ("of Dunbar" "AFTN" nil "AFDN")
      ("of England" "AFNK" nil)
      ("of Essex" "AFSK" nil)
      ("of Falaise" "AFFL" nil)
      ("of Flanders" "AFFL" nil)
      ("of Galloway" "AFKL" nil)
      ("of Germany" "AFKR" "AFJR")
      ("of Gloucester" "AFKL" nil)
      ("of Heristal" "AFRS" nil)
      ("of Hungary" "AFNK" nil)
      ("of Huntington" "AFNT" nil)
      ("of Kiev" "AFKF" nil "AFKV")
      ("of Kuno" "AFKN" nil)
      ("of Landen" "AFLN" nil)
      ("of Laon" "AFLN" nil)
      ("of Leinster" "AFLN" nil)
      ("of Lens" "AFLN" nil)
      ("of Lorraine" "AFLR" nil)
      ("of Louvain" "AFLF" nil "AFLV")
      ("of Mercia" "AFMR" nil)
      ("of Metz" "AFMT" nil)
      ("of Meulan" "AFML" nil)
      ("of Nass" "AFNS" nil)
      ("of Normandy" "AFNR" nil)
      ("of Ohningen" "AFNN" nil)
      ("of Orleans" "AFRL" nil)
      ("of Poitou" "AFPT" nil)
      ("of Polotzk" "AFPL" nil)
      ("of Provence" "AFPR" nil)
      ("of Ringelheim" "AFRN" nil)
      ("of Salisbury" "AFSL" nil)
      ("of Saxony" "AFSK" nil)
      ("of Scotland" "AFSK" nil)
      ("of Senlis" "AFSN" nil)
      ("of Stafford" "AFST" nil)
      ("of Swabia" "AFSP" nil "AFSB")
      ("of Tongres" "AFTN" nil)
      ("of the Tributes" "AF0T" "AFTT")
      ("unknown" "ANKN" nil)
      ("van der Gouda" "FNTR" nil "VNDR")
      ("von Adenbaugh" "FNTN" nil "VNDN")

      ;; some additional names to check out specific test cases

      ("Architure" "ARKT" nil)
      ("Arnoff" "ARNF" nil)
      ("Arnow" "ARN" "ARNF")
      ("Danger" "TNJR" "TNKR" "DNJR" "DNKR")
      ("Jankelowicz" "JNKL" "ANKL")
      ("Manger" "MNJR" "MNKR")
      ("McClellan" "MKLL" nil)
      ("McHugh" "MK" nil)
      ("McLaughlin" "MKLF" nil)
      ("Orchestra" "ARKS" nil)
      ("Orchid" "ARKT" nil "ARKD")
      ("Pierce" "PRS" nil)
      ("Ranger" "RNJR" "RNKR")
      ("Schlesinger" "XLSN" "SLSN")
      ("Uomo" "AM" nil)
      ("Vasserman" "FSRM" nil "VSRM")
      ("Wasserman" "ASRM" "FSRM")
      ("Womo" "AM" "FM")
      ("Yankelovich" "ANKL" nil)
      ("accede" "AKST" nil "AKSD")
      ("accident" "AKST" nil "AKSD")
      ("adelsheim" "ATLS" nil "ADLS")
      ("aged" "AJT" "AKT" "AJD" "AKD")
      ("ageless" "AJLS" "AKLS")
      ("agency" "AJNS" "AKNS")
      ("aghast" "AKST" nil)
      ("agio" "AJ" "AK")
      ("agrimony" "AKRM" nil)
      ("album" "ALPM" nil "ALBM")
      ("alcmene" "ALKM" nil)
      ("alehouse" "ALHS" nil)
      ("antique" "ANTK" nil)
      ("artois" "ART" "ARTS")
      ("automation" "ATMX" nil)
      ("bacchus" "PKS" nil "BKS")
      ("bacci" "PX" nil "BX")
      ("bajador" "PJTR" "PHTR" "BJDR" "BHDR")
      ("bellocchio" "PLX" nil "BLX")
      ("bertucci" "PRTX" nil "BRTX")
      ("biaggi" "PJ" "PK" "BJ" "BK")
      ("bough" "P" nil "B")
      ("breaux" "PR" nil "BR")
      ("broughton" "PRTN" nil "BRTN")
      ("cabrillo" "KPRL" "KPR" "KBRL" "KBR")
      ("caesar" "SSR" nil)
      ("cagney" "KKN" nil)
      ("campbell" "KMPL" nil "KMBL")
      ("carlisle" "KRLL" nil)
      ("carlysle" "KRLL" nil)
      ("chemistry" "KMST" nil)
      ("chianti" "KNT" nil)
      ("chorus" "KRS" nil)
      ("cough" "KF" nil)
      ("czerny" "SRN" "XRN")
      ("deffenbacher" "TFNP" nil "DFNB")
      ("dumb" "TM" nil "DM")
      ("edgar" "ATKR" nil)
      ("edge" "AJ" nil)
      ("filipowicz" "FLPT" "FLPF")
      ("focaccia" "FKX" nil)
      ("gallegos" "KLKS" "KKS")
      ("gambrelli" "KMPR" nil "KMBR")
      ("geithain" "K0N" "JTN")
      ("ghiradelli" "JRTL" nil "JRDL")
      ("ghislane" "JLN" nil)
      ("gough" "KF" nil)
      ("hartheim" "HR0M" "HRTM")
      ("heimsheim" "HMSM" nil)
      ("hochmeier" "HKMR" nil)
      ("hugh" "H" nil)
      ("hunger" "HNKR" "HNJR")
      ("hungry" "HNKR" nil)
      ("island" "ALNT" nil "ALND")
      ("isle" "AL" nil)
      ("jose" "HS" nil)
      ("laugh" "LF" nil)
      ("mac caffrey" "MKFR" nil)
      ("mac gregor" "MKRK" nil)
      ("pegnitz" "PNTS" "PKNT" "PNTZ")
      ("piskowitz" "PSKT" "PSKF")
      ("queen" "KN" nil)
      ("raspberry" "RSPR" nil "RSBR")
      ("resnais" "RSN" "RSNS")
      ("rogier" "RJ" "RJR")
      ("rough" "RF" nil)
      ("san jacinto" "SNHS" nil)
      ("schenker" "XNKR" "SKNK")
      ("schermerhorn" "XRMR" "SKRM")
      ("schmidt" "XMT" "SMT" "XMD" "SMD")
      ("schneider" "XNTR" "SNTR" "XNDR" "SNDR")
      ("school" "SKL" nil)
      ("schooner" "SKNR" nil)
      ("schrozberg" "XRSP" "SRSP" "XRZB" "SRZB")
      ("schulman" "XLMN" nil)
      ("schwabach" "XPK" "XFPK" "XBK" "XFBK")
      ("schwarzach" "XRSK" "XFRT")
      ("smith" "SM0" "XMT")
      ("snider" "SNTR" "XNTR" "SNDR" "XNDR")
      ("succeed" "SKST" nil "SKSD")
      ("sugarcane" "XKRK" "SKRK")
      ("svobodka" "SFPT" nil "SVBD")
      ("tagliaro" "TKLR" "TLR")
      ("thames" "TMS" nil)
      ("theilheim" "0LM" "TLM")
      ("thomas" "TMS" nil)
      ("thumb" "0M" "TM")
      ("tichner" "TXNR" "TKNR")
      ("tough" "TF" nil)
      ("umbrella" "AMPR" nil "AMBR")
      ("vilshofen" "FLXF" nil "VLXF")
      ("von schuller" "FNXL" nil "VNXL")
      ("wachtler" "AKTL" "FKTL")
      ("wechsler" "AKSL" "FKSL")
      ("weikersheim" "AKRS" "FKRS")
      ("zhao" "J" nil)))

;;; ---------------------------------------------------------------------------

(defparameter *basic-matches*
    '(("Arnow" "Arnoff")
      ("breaux" "brow")
      ("gnu" "new")
      ("thumb" "them")
      ("Uomo" "Womo")
      ("Wasserman" "Vasserman")
      ("Xavier" "savior")))

;;; ---------------------------------------------------------------------------
;;;  The following test pairs are translated from
;;;  http://aspell.sourceforge.net/test/batch0.tab
;;;
;;;  Copyright (C) 2002 Kevin Atkinson (kevina@gnu.org).
;;;  Verbatim copying and distribution is permitted in any medium, provided
;;;  this notice is preserved.

(defparameter *more-matches*
    '(("Accosinly" "Occasionally")
      ("Ciculer" "Circler")
      ("Circue" "Circle")
      ("Maddness" "Madness")
      ("Occusionaly" "Occasionally")
      ("Steffen" "Stephen")
      ("Thw" "The")
      ("Unformanlly" "Unfortunately")
      ("Unfortally" "Unfortunately")
      ("abilitey" "ability")
      ("abouy" "about")
      ("absorbtion" "absorption")
      ("accidently" "accidentally")
      ("accomodate" "accommodate")
      ("acommadate" "accommodate")
      ("acord" "accord")
      ("adultry" "adultery")
      ("aggresive" "aggressive")
      ("alchohol" "alcohol")
      ("alchoholic" "alcoholic")
      ("allieve" "alive")
      ("alot" "a lot")
      ("alright" "all right")
      ("amature" "amateur")
      ("ambivilant" "ambivalent")
      ("amification" "amplification")
      ("amourfous" "amorphous")
      ("annoint" "anoint")
      ("annonsment" "announcement")
      ("annoyting" "anting")
      ("annuncio" "announce")
      ("anonomy" "anatomy")
      ("anotomy" "anatomy")
      ("antidesestablishmentarianism" "antidisestablishmentarianism")
      ("antidisestablishmentarism" "antidisestablishmentarianism")
      ("anynomous" "anonymous")
      ("appelet" "applet")
      ("appreceiated" "appreciated")
      ("appresteate" "appreciate")
      ("aquantance" "acquaintance")
      ("aratictature" "architecture")
      ("archeype" "archetype")
      ("aricticure" "architecture")
      ("artic" "arctic")
      ("asentote" "asymptote")
      ("ast" "at")
      ("asterick" "asterisk")
      ("asymetric" "asymmetric")
      ("atentively" "attentively")
      ("autoamlly" "automatically")
      ("bankrot" "bankrupt")
      ("basicly" "basically")
      ("batallion" "battalion")
      ("bbrose" "browse")
      ("beauro" "bureau")
      ("beaurocracy" "bureaucracy")
      ("beggining" "beginning")
      ("beging" "beginning")
      ("behaviour" "behavior")
      ("beleive" "believe")
      ("belive" "believe")
      ("benidifs" "benefits")
      ("bigginging" "beginning")
      ("blait" "bleat")
      ("bouyant" "buoyant")
      ("boygot" "boycott")
      ("brocolli" "broccoli")
      ("buch" "bush")
      ("buder" "butter")
      ("budr" "butter")
      ("budter" "butter")
      ("buracracy" "bureaucracy")
      ("burracracy" "bureaucracy")
      ("buton" "button")
      ("byby" "by by")
      ("cauler" "caller")
      ("ceasar" "caesar")
      ("cemetary" "cemetery")
      ("changeing" "changing")
      ("cheet" "cheat")
      ("cicle" "circle")
      ("cimplicity" "simplicity")
      ("circumstaces" "circumstances")
      ("clob" "club")
      ("coaln" "colon")
      ("cocamena" "cockamamie")
      ("colleaque" "colleague")
      ("colloquilism" "colloquialism")
      ("columne" "column")
      ("comiler" "compiler")
      ("comitmment" "commitment")
      ("comitte" "committee")
      ("comittmen" "commitment")
      ("comittmend" "commitment")
      ("commerciasl" "commercials")
      ("commited" "committed")
      ("commitee" "committee")
      ("companys" "companies")
      ("compicated" "complicated")
      ("comupter" "computer")
      ("concensus" "consensus")
      ("confusionism" "confucianism")
      ("congradulations" "congratulations")
      ("conibation" "contribution")
      ("consident" "consistent")
      ("consident" "consonant")
      ("contast" "constant")
      ("contastant" "constant")
      ("contunie" "continue")
      ("cooly" "coolly")
      ("copping" "coping")
      ("cosmoplyton" "cosmopolitan")
      ("courst" "court")
      ("crasy" "crazy")
      ("cravets" "caveats")
      ("credetability" "credibility")
      ("criqitue" "critique")
      ("croke" "croak")
      ("crucifiction" "crucifixion")
      ("crusifed" "crucified")
      ("ctitique" "critique")
      ("cumba" "combo")
      ("custamisation" "customization")
      ("dag" "dog")
      ("daly" "daily")
      ("danguages" "dangerous")
      ("deaft" "draft")
      ("defence" "defense")
      ("defenly" "defiantly")
      ("definate" "definite")
      ("definately" "definitely")
      ("dependeble" "dependable")
      ("descrption" "description")
      ("descrptn" "description")
      ("desparate" "desperate")
      ("dessicate" "desiccate")
      ("destint" "distant")
      ("develepment" "developments")
      ("developement" "development")
      ("develpond" "development")
      ("devulge" "divulge")
      ("diagree" "disagree")
      ("dieties" "deities")
      ("dinasaur" "dinosaur")
      ("dinasour" "dinosaur")
      ("direcyly" "directly")
      ("discuess" "discuss")
      ("disect" "dissect")
      ("disippate" "dissipate")
      ("disition" "decision")
      ("dispair" "despair")
      ("disssicion" "discussion")
      ("distarct" "distract")
      ("distart" "distort")
      ("distroy" "destroy")
      ("documtations" "documentation")
      ("doenload" "download")
      ("dongle" "dangle")
      ("doog" "dog")
      ("dramaticly" "dramatically")
      ("drunkeness" "drunkenness")
      ("ductioneery" "dictionary")
      ("dur" "due")
      ("duren" "during")
      ("dymatic" "dynamic")
      ("dynaic" "dynamic")
      ("ecstacy" "ecstasy")
      ("efficat" "efficient")
      ("efficity" "efficacy")
      ("effots" "efforts")
      ("egsistence" "existence")
      ("eitiology" "etiology")
      ("elagent" "elegant")
      ("elligit" "elegant")
      ("embarass" "embarrass")
      ("embarassment" "embarrassment")
      ("embaress" "embarrass")
      ("encapsualtion" "encapsulation")
      ("encyclapidia" "encyclopedia")
      ("encyclopia" "encyclopedia")
      ("engins" "engine")
      ("enhence" "enhance")
      ("enligtment" "Enlightenment")
      ("ennuui" "ennui")
      ("enought" "enough")
      ("enventions" "inventions")
      ("envireminakl" "environmental")
      ("enviroment" "environment")
      ("epitomy" "epitome")
      ("equire" "acquire")
      ("errara" "error")
      ("erro" "error")
      ("evaualtion" "evaluation")
      ("evething" "everything")
      ("evtually" "eventually")
      ("excede" "exceed")
      ("excercise" "exercise")
      ("excpt" "except")
      ("excution" "execution")
      ("exhileration" "exhilaration")
      ("existance" "existence")
      ("expleyly" "explicitly")
      ("explity" "explicitly")
      ("expresso" "espresso")
      ("exspidient" "expedient")
      ("extions" "extensions")
      ("factontion" "factorization")
      ("failer" "failure")
      ("famdasy" "fantasy")
      ("faver" "favor")
      ("faxe" "fax")
      ("febuary" "february")
      ("firey" "fiery")
      ("fistival" "festival")
      ("flatterring" "flattering")
      ("fluk" "flux")
      ("flukse" "flux")
      ("fone" "phone")
      ("forsee" "foresee")
      ("frustartaion" "frustrating")
      ("fuction" "function")
      ("funetik" "phonetic")
      ("futs" "guts")
      ("gamne" "came")
      ("gaurd" "guard")
      ("generly" "generally")
      ("ghandi" "gandhi")
      ("goberment" "government")
      ("gobernement" "government")
      ("gobernment" "government")
      ("gotton" "gotten")
      ("gracefull" "graceful")
      ("gradualy" "gradually")
      ("grammer" "grammar")
      ("hallo" "hello")
      ("hapily" "happily")
      ("harrass" "harass")
      ("havne" "have")
      ("heellp" "help")
      ("heighth" "height")
      ("hellp" "help")
      ("helo" "hello")
      ("herlo" "hello")
      ("hifin" "hyphen")
      ("hifine" "hyphen")
      ("higer" "higher")
      ("hiphine" "hyphen")
      ("hippie" "hippy")
      ("hippopotamous" "hippopotamus")
      ("hlp" "help")
      ("hourse" "horse")
      ("houssing" "housing")
      ("howaver" "however")
      ("howver" "however")
      ("humaniti" "humanity")
      ("hyfin" "hyphen")
      ("hypotathes" "hypothesis")
      ("hypotathese" "hypothesis")
      ("hystrical" "hysterical")
      ("ident" "indent")
      ("illegitament" "illegitimate")
      ("imbed" "embed")
      ("imediaetly" "immediately")
      ("imfamy" "infamy")
      ("immenant" "immanent")
      ("implemtes" "implements")
      ("inadvertant" "inadvertent")
      ("incase" "in case")
      ("incedious" "insidious")
      ("incompleet" "incomplete")
      ("incomplot" "incomplete")
      ("inconvenant" "inconvenient")
      ("inconvience" "inconvenience")
      ("independant" "independent")
      ("independenent" "independent")
      ("indepnends" "independent")
      ("indepth" "in depth")
      ("indispensible" "indispensable")
      ("inefficite" "inefficient")
      ("inerface" "interface")
      ("infact" "in fact")
      ("influencial" "influential")
      ("inital" "initial")
      ("initinized" "initialized")
      ("initized" "initialized")
      ("innoculate" "inoculate")
      ("insistant" "insistent")
      ("insistenet" "insistent")
      ("instulation" "installation")
      ("intealignt" "intelligent")
      ("intejilent" "intelligent")
      ("intelegent" "intelligent")
      ("intelegnent" "intelligent")
      ("intelejent" "intelligent")
      ("inteligent" "intelligent")
      ("intelignt" "intelligent")
      ("intellagant" "intelligent")
      ("intellegent" "intelligent")
      ("intellegint" "intelligent")
      ("intellgnt" "intelligent")
      ("intensionality" "intensionally")
      ("interate" "iterate")
      ("internation" "international")
      ("interpretate" "interpret")
      ("interpretter" "interpreter")
      ("intertes" "interested")
      ("intertesd" "interested")
      ("invermeantial" "environmental")
      ("irregardless" "regardless")
      ("irresistable" "irresistible")
      ("irritible" "irritable")
      ("islams" "muslims")
      ("isotrop" "isotope")
      ("isreal" "israel")
      ("johhn" "john")
      ("judgement" "judgment")
      ("kippur" "kipper")
      ("knawing" "knowing")
      ("latext" "latest")
      ("leasve" "leave")
      ("lesure" "leisure")
      ("liasion" "lesion")
      ("liason" "liaison")
      ("libary" "library")
      ("likly" "likely")
      ("lilometer" "kilometer")
      ("liquify" "liquefy")
      ("lloyer" "layer")
      ("lossing" "losing")
      ("luser" "laser")
      ("maintanence" "maintenance")
      ("majaerly" "majority")
      ("majoraly" "majority")
      ("maks" "masks")
      ("mandelbrot" "Mandelbrot")
      ("mant" "want")
      ("marshall" "marshal")
      ("maxium" "maximum")
      ("meory" "memory")
      ("metter" "better")
      ("mic" "mike")
      ("midia" "media")
      ("millenium" "millennium")
      ("miniscule" "minuscule")
      ("minkay" "monkey")
      ("minum" "minimum")
      ("mischievious" "mischievous")
      ("misilous" "miscellaneous")
      ("momento" "memento")
      ("monkay" "monkey")
      ("mosaik" "mosaic")
      ("mostlikely" "most likely")
      ("mousr" "mouser")
      ("mroe" "more")
      ("neccessary" "necessary")
      ("necesary" "necessary")
      ("necesser" "necessary")
      ("neice" "niece")
      ("neighbour" "neighbor")
      ("nemonic" "pneumonic")
      ("nevade" "Nevada")
      ("nickleodeon" "nickelodeon")
      ("nieve" "naive")
      ("noone" "no one")
      ("noticably" "noticeably")
      ("notin" "not in")
      ("nozled" "nuzzled")
      ("objectsion" "objects")
      ("obsfuscate" "obfuscate")
      ("ocassion" "occasion")
      ("occuppied" "occupied")
      ("occurence" "occurrence")
      ("octagenarian" "octogenarian")
      ("olf" "old")
      ("opposim" "opossum")
      ("organise" "organize")
      ("organiz" "organize")
      ("orientate" "orient")
      ("oscilascope" "oscilloscope")
      ("oving" "moving")
      ("paramers" "parameters")
      ("parametic" "parameter")
      ("paranets" "parameters")
      ("partrucal" "particular")
      ("pataphysical" "metaphysical")
      ("patten" "pattern")
      ("permissable" "permissible")
      ("permition" "permission")
      ("permmasivie" "permissive")
      ("perogative" "prerogative")
      ("persue" "pursue")
      ("phantasia" "fantasia")
      ("phenominal" "phenomenal")
      ("picaresque" "picturesque")
      ("playwrite" "playwright")
      ("poeses" "poesies")
      ("polation" "politician")
      ("poligamy" "polygamy")
      ("politict" "politic")
      ("pollice" "police")
      ("polypropalene" "polypropylene")
      ("pompom" "pompon")
      ("possable" "possible")
      ("practicle" "practical")
      ("pragmaticism" "pragmatism")
      ("preceeding" "preceding")
      ("precion" "precision")
      ("precios" "precision")
      ("preemptory" "peremptory")
      ("prefices" "prefixes")
      ("prefixt" "prefixed")
      ("presbyterian" "Presbyterian")
      ("presue" "pursue")
      ("presued" "pursued")
      ("privielage" "privilege")
      ("priviledge" "privilege")
      ("proceedures" "procedures")
      ("pronensiation" "pronunciation")
      ("pronisation" "pronunciation")
      ("pronounciation" "pronunciation")
      ("properally" "properly")
      ("proplematic" "problematic")
      ("protray" "portray")
      ("pscolgst" "psychologist")
      ("psicolagest" "psychologist")
      ("psycolagest" "psychologist")
      ("quoz" "quiz")
      ("radious" "radius")
      ("ramplily" "rampantly")
      ("reccomend" "recommend")
      ("reccona" "raccoon")
      ("recieve" "receive")
      ("reconise" "recognize")
      ("rectangeles" "rectangle")
      ("redign" "redesign")
      ("reoccurring" "recurring")
      ("repitition" "repetition")
      ("replasments" "replacement")
      ("reposable" "responsible")
      ("reseblence" "resemblance")
      ("respct" "respect")
      ("respecally" "respectfully")
      ("roon" "room")
      ("rought" "roughly")
      ("rsx" "RSX")
      ("rudemtry" "rudimentary")
      ("runnung" "running")
      ("sacreligious" "sacrilegious")
      ("saftly" "safely")
      ("salut" "salute")
      ("satifly" "satisfy")
      ("scrabdle" "scrabble")
      ("searcheable" "searchable")
      ("secion" "section")
      ("seferal" "several")
      ("segements" "segments")
      ("sence" "sense")
      ("seperate" "separate")
      ("sherbert" "sherbet")
      ("sicolagest" "psychologist")
      ("sieze" "seize")
      ("simpfilty" "simplicity")
      ("simplye" "simply")
      ("singal" "signal")
      ("sitte" "site")
      ("situration" "situation")
      ("slyph" "sylph")
      ("smil" "smile")
      ("snuck" "sneaked")
      ("sometmes" "sometimes")
      ("soonec" "sonic")
      ("specificialy" "specifically")
      ("spel" "spell")
      ("spoak" "spoke")
      ("sponsered" "sponsored")
      ("stering" "steering")
      ("straightjacket" "straitjacket")
      ("stumach" "stomach")
      ("stutent" "student")
      ("styleguide" "style guide")
      ("subisitions" "substitutions")
      ("subjecribed" "subscribed")
      ("subpena" "subpoena")
      ("substations" "substitutions")
      ("suger" "sugar")
      ("supercede" "supersede")
      ("superfulous" "superfluous")
      ("susan" "Susan")
      ("swimwear" "swim wear")
      ("syncorization" "synchronization")
      ("taff" "tough")
      ("taht" "that")
      ("tattos" "tattoos")
      ("techniquely" "technically")
      ("teh" "the")
      ("tem" "team")
      ("teo" "two")
      ("teridical" "theoretical")
      ("tesst" "test")
      ("tets" "tests")
      ("thanot" "than or")
      ("theirselves" "themselves")
      ("theridically" "theoretical")
      ("thredically" "theoretically")
      ("thruout" "throughout")
      ("ths" "this")
      ("titalate" "titillate")
      ("tobagan" "tobaggon")
      ("tommorrow" "tomorrow")
      ("tomorow" "tomorrow")
      ("tradegy" "tragedy")
      ("trubbel" "trouble")
      ("ttest" "test")
      ("tunnellike" "tunnel like")
      ("tured" "turned")
      ("tyrrany" "tyranny")
      ("unatourral" "unnatural")
      ("unaturral" "unnatural")
      ("unconisitional" "unconstitutional")
      ("unconscience" "unconscious")
      ("underladder" "under ladder")
      ("unentelegible" "unintelligible")
      ("unfortunently" "unfortunately")
      ("unnaturral" "unnatural")
      ("upcast" "up cast")
      ("upmost" "utmost")
      ("uranisium" "uranium")
      ("verison" "version")
      ("vinagarette" "vinaigrette")
      ("volumptuous" "voluptuous")
      ("volunteerism" "voluntarism")
      ("volye" "volley")
      ("wadting" "wasting")
      ("waite" "wait")
      ("wan't" "won't")
      ("warloord" "warlord")
      ("whaaat" "what")
      ("whard" "ward")
      ("whimp" "wimp")
      ("wicken" "weaken")
      ("wierd" "weird")
      ("wrank" "rank")
      ("writeen" "righten")
      ("writting" "writing")
      ("wundeews" "windows")
      ("yeild" "yield")
      ("youe" "your")))

;;; ---------------------------------------------------------------------------

(defun report-problem (&rest args)
  (declare (dynamic-extent args))
  (apply #'format t "~&;;     ~@?" args))	   

;;; ---------------------------------------------------------------------------

(defun check-encodings (&optional extended-p)
  (format t "~&;;   Starting encodings test~@[ (testing both strict & ~
                    extended-p)~]...~%" 
          extended-p)
  (loop for test in *encodings* do
        (destructuring-bind (string expected-primary expected-secondary
                             &optional expected-extended-primary
                                       expected-extended-secondary)
            test
          (multiple-value-bind (primary secondary)
              (double-metaphone string)
            (unless (string= primary expected-primary)
              (report-problem 
               "Primary of ~s should be ~s, got ~s"
               string expected-primary primary))
            (unless (or (and (not expected-secondary) (not secondary))
                        (string= secondary expected-secondary)
                        (report-problem 
                         "Secondary of ~s should be ~s, got ~s"
                         string expected-secondary secondary)))
            (when extended-p
              (multiple-value-bind (extended-primary extended-secondary)
                  (double-metaphone string 't)
                (unless (if expected-extended-primary
                            (string= extended-primary 
                                     expected-extended-primary)
                            (string= expected-primary extended-primary))
                  (report-problem 
                   "Extended primary of ~s should be ~s, got ~s"
                   string expected-extended-primary extended-primary))
                (unless (if expected-extended-secondary
                            (string= extended-secondary 
                                     expected-extended-secondary)
                            (string= secondary extended-secondary))
                  (report-problem 
                   "Extended secondary of ~s should be ~s, got ~s"
                   string expected-extended-secondary extended-secondary)))))))
  (format t "~&;;   Encodings test completed.~%"))

;;; ---------------------------------------------------------------------------

(defun check-the-matches (matches label summarize-p &optional extended-p)
  (format t "~&;;   Starting ~a matches test~@[ (summarizing)~]...~%" 
	  label
	  summarize-p)
  (let ((total-count 0)
	(strong-match-count 0)
	(normal-match-count 0)
	(weak-match-count 0)
	(failure-count 0))
    (loop for (string1 string2) in matches do
	  (incf& total-count)
	  (multiple-value-bind (primary1 secondary1)
	      (double-metaphone string1 extended-p)
	    (multiple-value-bind (primary2 secondary2)
		(double-metaphone string2 extended-p)
	      (unless (or
		       ;; Strong match:
		       (when (string= primary1 primary2)
			 (incf& strong-match-count))
		       ;; Normal matches:
		       (when (or (and secondary2 (string= primary1 secondary2))
				 (and secondary1 (string= secondary1 primary2)))
			 (incf& normal-match-count))
		       ;; Weak match:
		       (when (and secondary1 
				  secondary2 
				  (string= secondary1 secondary2))
			 (incf& weak-match-count)))
		(incf& failure-count)
		(unless summarize-p
		  (report-problem
		   "Match of ~s and ~s failed (~s~@[;~s~] ~s~@[;~s~])."
		   string1 string2 
		   primary1 secondary1
		   primary2 secondary2))))))
    (format t "~&;;   ~@(~a matches~) test completed~@[ using extended-p~*~] ~
                      (~s/~s S:~s N:~s W:~s).~%"
	    label
            extended-p
	    (-& total-count failure-count)
	    total-count
	    strong-match-count
	    normal-match-count
	    weak-match-count)))

;;; ---------------------------------------------------------------------------

(defun check-basic-matches (&optional summarize-p extended-p)
  (check-the-matches *basic-matches* "basic" summarize-p extended-p))

(defun check-more-matches (&optional summarize-p extended-p)
  (check-the-matches *more-matches* "more" summarize-p extended-p))

;;; ---------------------------------------------------------------------------

(defun double-metaphone-tests ()
  (format t "~&;; Starting double-metaphone tests...~%")
  (check-encodings 't)
  (check-basic-matches 't)
  (check-basic-matches 't 't)
  (check-more-matches 't)
  (check-more-matches 't 't)
  (format t "~&;; Double-metaphone tests completed.~%"))

;;; ---------------------------------------------------------------------------

(when *autorun-modules* 
  (time (double-metaphone-tests)))

;;; ===========================================================================
;;;                               End of File
;;; ===========================================================================
